@@ -162,6 +162,8 @@ static int	errordata_stack_depth = -1; /* index of topmost active frame */
 
 static int	recursion_depth = 0;	/* to detect actual recursion */
 
+static int	frontend_error_recursion_depth = 0;	/* to detect recursion in delivering error to frontend clients */
+
 
 /* Macro for checking errordata_stack_depth is reasonable */
 #define CHECK_STACK_DEPTH() \
@@ -215,11 +217,17 @@ errstart(int elevel, const char *filename, int lineno,
 	bool		output_to_server;
 	bool		output_to_client = false;
 	int			i;
-
+	int			frontend_invalid = false;
 	/*
 	 * Check some cases in which we want to promote an error into a more
 	 * severe error.  None of this logic applies for non-error messages.
 	 */
+	if (elevel == FRONTEND_ERROR)
+	{
+		frontend_invalid = true;
+		elevel = ERROR;
+	}
+
 	if (elevel >= ERROR)
 	{
 		/*
@@ -301,6 +309,7 @@ errstart(int elevel, const char *filename, int lineno,
 		if (in_error_recursion_trouble())
 		{
 			error_context_stack = NULL;
+			output_to_client = false;
 		}
 	}
 	if (++errordata_stack_depth >= ERRORDATA_STACK_SIZE)
@@ -318,6 +327,7 @@ errstart(int elevel, const char *filename, int lineno,
 	edata = &errordata[errordata_stack_depth];
 	MemSet(edata, 0, sizeof(ErrorData));
 	edata->elevel = elevel;
+	edata->frontend_invalid = frontend_invalid;
 	edata->output_to_server = output_to_server;
 	edata->output_to_client = output_to_client;
 	if(elevel == FATAL && PG_exception_stack == NULL) /* This is startup failure. Take down main process with it */
@@ -530,15 +540,6 @@ int pool_error_code(const char *errcode)
 	return 0;					/* return value does not matter */
 }
 
-/*
- * errcode_for_socket_access --- add SQLSTATE error code to the current error
- *
- * The SQLSTATE code is chosen based on the saved errno value.	We assume
- * that the failing operation was some type of socket access.
- *
- * NOTE: the primary error message string should generally include %m
- * when this is used.
- */
 
 /*
  * This macro handles expansion of a format string and associated parameters;
@@ -965,6 +966,21 @@ geterrcode(void)
 	return edata->sqlerrcode;
 }
 
+/*
+ * getfrontendinvalid --- return the currently frontend_invalid value
+ *
+ * This is only intended for use in error callback subroutines, since there
+ * is no other place outside elog.c where the concept is meaningful.
+ */
+bool getfrontendinvalid(void)
+{
+	ErrorData  *edata = &errordata[errordata_stack_depth];
+
+	/* we don't bother incrementing recursion_depth */
+	CHECK_STACK_DEPTH();
+
+	return edata->frontend_invalid;
+}
 /*
  * geterrposition --- return the currently set error position (0 if none)
  *
@@ -1224,6 +1240,7 @@ FlushErrorState(void)
 	 */
 	errordata_stack_depth = -1;
 	recursion_depth = 0;
+	frontend_error_recursion_depth = 0;
 	/* Delete all data in ErrorContext */
 	MemoryContextResetAndDeleteChildren(ErrorContext);
 }
@@ -1576,7 +1593,7 @@ write_eventlog(int level, const char *line, int len)
 
 	if (evtHandle == INVALID_HANDLE_VALUE)
 	{
-		evtHandle = RegisterEventSource(NULL, event_source ? event_source : "PostgreSQL");
+		evtHandle = RegisterEventSource(NULL, event_source ? event_source : "pgpool");
 		if (evtHandle == NULL)
 		{
 			evtHandle = INVALID_HANDLE_VALUE;
@@ -1735,6 +1752,11 @@ send_message_to_frontend(ErrorData *edata)
 	 */
 	if (edata->elevel < ERROR && pool_get_session_context(true) == NULL)
 		return;
+	/*
+	 * Leave if we are failing on sending the message to frontend.
+	 */
+	if (++frontend_error_recursion_depth > 2)
+		return;
 
 	protoVersion = get_frontend_protocol_version();
 	set_pg_frontend_blocking(false);
@@ -1829,6 +1851,7 @@ send_message_to_frontend(ErrorData *edata)
 	}
 
 	set_pg_frontend_blocking(true);
+	frontend_error_recursion_depth--;
 }
 
 /*
